/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Basic text widget and supporting code for widgets containing text.
 *
 * All content widgets have one (at least for basic content widgets) Content, from
 * mde.gui.content.Items . */
module mde.gui.widget.TextWidget;

import mde.gui.widget.Widget;
import mde.gui.exception;
import mde.gui.renderer.IRenderer;
import mde.gui.content.Content;

import mde.font.font;

/// Basic text widget
class TextLabelWidget : Widget
{
    /** Constructor for a widget containing [fixed] content.
     *
     * Widget uses the initialisation data:
     * [widgetID, contentID, colour]
     * where contentID is an ID for the string ID of the contained content
     * and colour is an 8-bit-per-channel RGB colour of the form 0xRRGGBB. */
    this (IWidgetManager mgr, WidgetData data) {
        WDCheck (data, 2, 1);
        if (font is null) font = FontStyle.get("default");
        font.updateBlock (data.strings[0], textCache);
        mw = cast(wdim) textCache.w;
        mh = cast(wdim) textCache.h;
        colour = Colour (data.ints[1]);
        super (mgr,data);
    }
    
    void draw () {
        super.draw();
        font.textBlock (x,y, text, textCache, colour);
    }
    
protected:
    char[] text;
    Colour colour;
    TextBlock textCache;
    static FontStyle font;
}


/// Adapter to ease use of ContentLabelWidget
struct ContentLabelAdapter {
    void set (IContent c, int col) {
        if (font is null) font = FontStyle.get("default");
        
        content = c;
        colour = Colour (cast(ubyte) (col >> 16u),
                         cast(ubyte) (col >> 8u),
                         cast(ubyte) col );
    }
    
    void getDimensions (out wdsize w, out wdsize h) {
        font.updateBlock (content.toString, textCache);
        w = cast(wdim) textCache.w;
        h = cast(wdim) textCache.h;
    }
    
    void draw (wdabs x, wdabs y) {
        font.textBlock (x,y, content.toString, textCache, colour);
    }
    
    IContent content;
    TextBlock textCache;
    Colour colour;
    static FontStyle font;
}

/// Basic widget displaying a label from a content.
class ContentLabelWidget : Widget
{
    this (IWidgetManager mgr, WidgetData data, IContent c) {
        WDCheck (data, 2, 0);
        adapter.set (c, data.ints[1]);
        adapter.getDimensions (mw, mh);
        super (mgr,data);
    }
    
    void draw () {
        super.draw();
        adapter.draw (x,y);
    }
    
protected:
    ContentLabelAdapter adapter;
}
